@@ -1,8 +1,11 @@
 from datetime import date
 
 import click
+import numpy as np
 from pandas import concat
 from pandas import DataFrame
+from pandas import json_normalize
+from pandas import Series
 from pandas import to_datetime
 from py42 import exceptions
 from py42.exceptions import Py42NotFoundError
@@ -19,6 +22,7 @@ from code42cli.file_readers import read_csv_arg
 from code42cli.options import format_option
 from code42cli.options import sdk_options
 from code42cli.output_formats import DataFrameOutputFormatter
+from code42cli.output_formats import OutputFormat
 from code42cli.output_formats import OutputFormatter
 
 
@@ -151,11 +155,11 @@ def _change_device_name(sdk, guid, name):
 @devices.command()
 @device_guid_argument
 @sdk_options()
-def show(state, device_guid, format=None):
+def show(state, device_guid):
     """Print individual device details. Requires device GUID."""
 
-    formatter = OutputFormatter(format, _device_info_keys_map())
-    backup_set_formatter = OutputFormatter(format, _backup_set_keys_map())
+    formatter = OutputFormatter(OutputFormat.TABLE, _device_info_keys_map())
+    backup_set_formatter = OutputFormatter(OutputFormat.TABLE, _backup_set_keys_map())
     device_info = _get_device_info(state.sdk, device_guid)
     formatter.echo_formatted_list([device_info])
     backup_usage = device_info.get("backupUsage")
@@ -244,6 +248,22 @@ include_usernames_option = click.option(
     is_flag=True,
     help="Include device settings in output.",
 )
+@click.option(
+    "--include-legal-hold-membership",
+    required=False,
+    type=bool,
+    default=False,
+    is_flag=True,
+    help="Include legal hold membership in output.",
+)
+@click.option(
+    "--include-total-storage",
+    required=False,
+    type=bool,
+    default=False,
+    is_flag=True,
+    help="Include backup archive count and total storage in output.",
+)
 @click.option(
     "--exclude-most-recently-connected",
     type=int,
@@ -285,6 +305,8 @@ def list_devices(
     include_backup_usage,
     include_usernames,
     include_settings,
+    include_legal_hold_membership,
+    include_total_storage,
     exclude_most_recently_connected,
     last_connected_after,
     last_connected_before,
@@ -309,7 +331,11 @@ def list_devices(
         "userUid",
     ]
     df = _get_device_dataframe(
-        state.sdk, columns, active, org_uid, include_backup_usage
+        state.sdk,
+        columns,
+        active,
+        org_uid,
+        (include_backup_usage or include_total_storage),
     )
     if last_connected_after:
         df = df.loc[to_datetime(df.lastConnected) > last_connected_after]
@@ -326,10 +352,14 @@ def list_devices(
             .head(exclude_most_recently_connected)
         )
         df = df.drop(most_recent.index)
+    if include_total_storage:
+        df = _add_storage_totals_to_dataframe(df, include_backup_usage)
     if include_settings:
         df = _add_settings_to_dataframe(state.sdk, df)
     if include_usernames:
         df = _add_usernames_to_device_dataframe(state.sdk, df)
+    if include_legal_hold_membership:
+        df = _add_legal_hold_membership_to_device_dataframe(state.sdk, df)
     if df.empty:
         click.echo("No results found.")
     else:
@@ -337,6 +367,42 @@ def list_devices(
         formatter.echo_formatted_dataframe(df)
 
 
+def _add_legal_hold_membership_to_device_dataframe(sdk, df):
+    columns = ["legalHold.legalHoldUid", "legalHold.name", "user.userUid"]
+
+    legal_hold_member_dataframe = (
+        json_normalize(list(_get_all_active_hold_memberships(sdk)))[columns]
+        .groupby(["user.userUid"])
+        .agg(",".join)
+        .rename(
+            {
+                "legalHold.legalHoldUid": "legalHoldUid",
+                "legalHold.name": "legalHoldName",
+            },
+            axis=1,
+        )
+    )
+    df = df.merge(
+        legal_hold_member_dataframe,
+        how="left",
+        left_on="userUid",
+        right_on="user.userUid",
+    )
+
+    df.loc[df["status"] == "Deactivated", ["legalHoldUid", "legalHoldName"]] = np.nan
+
+    return df
+
+
+def _get_all_active_hold_memberships(sdk):
+    for page in sdk.legalhold.get_all_matters(active=True):
+        for matter in page["legalHolds"]:
+            for _page in sdk.legalhold.get_all_matter_custodians(
+                legal_hold_uid=matter["legalHoldUid"], active=True
+            ):
+                yield from _page["legalHoldMemberships"]
+
+
 def _get_device_dataframe(
     sdk, columns, active=None, org_uid=None, include_backup_usage=False
 ):
@@ -392,6 +458,26 @@ def _add_usernames_to_device_dataframe(sdk, device_dataframe):
     return device_dataframe.merge(users_dataframe, how="left", on="userUid")
 
 
+def _add_storage_totals_to_dataframe(df, include_backup_usage):
+    df[["archiveCount", "totalStorageBytes"]] = df["backupUsage"].apply(
+        _break_backup_usage_into_total_storage
+    )
+
+    if not include_backup_usage:
+        df = df.drop("backupUsage", axis=1)
+    return df
+
+
+def _break_backup_usage_into_total_storage(backup_usage):
+    total_storage = 0
+    archive_count = 0
+    for archive in backup_usage:
+        if archive["archiveFormat"] != "ARCHIVE_V2":
+            archive_count += 1
+            total_storage += archive["archiveBytes"]
+    return Series([archive_count, total_storage])
+
+
 @devices.command()
 @active_option
 @inactive_option
